import React from "react";
import { connect } from 'react-redux'
import { addUser, removeUser } from '../store/users.slice'
import { useParams } from 'react-router-dom';

// wrap component so that it can use route params within this.props
function withParams(Component) {
  return props => <Component {...props} params={useParams()} />;
}

class UsersWithoutHooks extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      firstName: '',
      lastName: '',
      age: 0
    };
    // liaison entre handleAdd() et this => permet à handleAdd() d'être appelé
    // hors du contexte de cet objet, donc quand this ne correspond pas cet objet.
    this.handleAdd = this.handleAdd.bind(this);
    // Dans cet exemple, lier handleRemove() est inutile car la fonction est
    // uniquement appelée grâce à une fonction fléchée, qui lie par défaut l'appel de la méthode
    // au contexte de cet objet. La méthode a donc un this qui correspond bien à cet objet.
    this.handleRemove = this.handleRemove.bind(this);
    // log() devrait être relié => un warning s'affiche si on appelle log() sans passer par une
    // fonction fléchée, comme c'est le cas avec le bouton du bas.
  }

  handleAdd() {
    if ((this.state.firstName !== "") && (this.state.lastName !== "")) {
      this.props.addUser({firstName: this.state.firstName, lastName: this.state.lastName, age: this.state.age})
    }
  }

  handleRemove(id) {
    this.props.removeUser(id)
  }

  log() {
    console.log(this.state)
  }

  render() {
    return (
      <div>
        <p>Users : {this.props.msg}</p>
        <hr />
        <p>{this.props.params.subtitle}</p>
        <hr/>
        first Name: <input value={this.state.firstName} onChange={(event)=> this.setState({firstName: event.target.value})} />
        last Name: <input value={this.state.lastName} onChange={(event)=> this.setState({lastName :event.target.value})} />
        age: <input value={this.state.age} onChange={(event)=> this.setState({age: event.target.value})} />
        <button onClick={this.handleAdd}>Add new user</button>
        <hr/>
        <ul>
          {this.props.usersTab.map((e,id) => <li key={id}>{e.firstName} {e.lastName} {e.age} <button onClick={() => this.handleRemove(id)}>Remove</button></li>)}
        </ul>
        <hr/>
        <button onClick={this.log}>Log current state</button>
      </div>
    )
  }
}

// pour relier redux à ce composant, il faut passer par connect()

// mapper les actions du store utilisées dans ce composant
const mapDispatchToProps = {
  addUser,
  removeUser
}
// mapper le state du slice utilisé dans ce composant
const mapStateToProps = (state) => {
  return state.users
}
// connecter ce composant à redux
export default connect(mapStateToProps, mapDispatchToProps)(withParams(UsersWithoutHooks))