import { createSlice, createAsyncThunk } from '@reduxjs/toolkit'
import {getAllTowns} from "../services/towns.service";

export const getTowns = createAsyncThunk('towns/getTowns', async () => {
  const response = await getAllTowns()
  return response
})

export const townsSlice = createSlice({
  name: 'towns',
  initialState: {
    townsTab: [],
    status: 'idle',
    error: null,
  },
  reducers: {
    addTown: (state, action) => {
      state.townsTab.push(action.payload)
    },
    removeTown: (state, action) => {
      let index = parseInt(action.payload)
      if (isNaN(index)) return
      state.townsTab.splice(index,1)
    },
  },
  extraReducers(builder) {
    builder
      .addCase(getTowns.pending, (state, action) => {
        state.status = 'loading'
      })
      .addCase(getTowns.fulfilled, (state, action) => {
        if (action.payload.error !== 0) {
          console.log('failure while getting the list of towns, got nothing')
          state.status = 'idle'
        }
        else {
          state.status = 'succeeded'

          // Add any fetched posts to the array
          state.townsTab.splice(0)
          state.townsTab = action.payload.data
        }
      })
      .addCase(getTowns.rejected, (state, action) => {
        state.status = 'failed'
        state.error = 'Abnormal case while getting the list of towns'
      })
  },
})

// Action creators are generated for each case reducer function
export const { addTown, removeTown } = townsSlice.actions
export default townsSlice.reducer